@@ -2,7 +2,7 @@
 #
 # django-codenerix-pos
 #
-# Copyright 2017 Centrologic Computational Logistic Center S.L.
+# Codenerix GNU
 #
 # Project URL : http://www.codenerix.com
 #
@@ -22,11 +22,14 @@ import json
 import hashlib
 import requests
 
+from django.contrib.auth.decorators import login_required
+from django.db import IntegrityError, transaction
 from django.db.models import Q
-from django.db.models.fields import FieldDoesNotExist
+from django.core.exceptions import FieldDoesNotExist
 from django.forms.utils import ErrorList
 from django.http import HttpResponse
-from django.utils.translation import ugettext as _
+from django.utils.decorators import method_decorator
+from django.utils.translation import gettext as _
 from django.views.generic import View
 from django.db.models import Count
 from django.conf import settings
@@ -34,8 +37,9 @@ from django.conf import settings
 from codenerix.views import GenList, GenCreate, GenCreateModal, GenUpdate, GenUpdateModal, GenDelete, GenDetail, GenDetailModal, GenForeignKey
 from codenerix_extensions.views import GenCreateBridge, GenUpdateBridge
 
-from .models import POSZone, POSHardware, POS, POSSlot, POSPlant, POSProduct, POSLog, POSOperator
-from .forms import POSZoneForm, POSHardwareForm, POSForm, POSSlotForm, POSPlantForm, POSProductForm, POSOperatorForm
+from .models import POSZone, POSHardware, POS, POSSlot, POSPlant, POSProduct, POSLog, POSOperator, POSGroupProduct
+from .models import KIND_POSHARDWARE_CASH
+from .forms import POSZoneForm, POSHardwareForm, POSForm, POSSlotForm, POSPlantForm, POSProductForm, POSProductFormGroup, POSOperatorForm, POSGroupProductForm
 
 
 # ###########################################
@@ -300,7 +304,7 @@ class POSCommits(GenForeignKey):
         answer.append({'id': 'LATEST', 'label': _('Latest configuration')})
 
         # Get all hashes
-        url = "https://api.github.com/repos/centrologic/django-codenerix-pos-client/commits"
+        url = "https://api.github.com/repos/codenerix/django-codenerix-pos-client/commits"
         r = requests.get(url, params={})
         if not r.raise_for_status():
             # Read the answer$
@@ -315,7 +319,7 @@ class POSCommits(GenForeignKey):
                         break
 
         # Add one last option saying it has more
-        if len(data)>10:
+        if len(data) > 10:
             answer.append({'id': None, 'label': '...'})
 
         # Convert the answer to JSON
@@ -378,12 +382,32 @@ class POSSlotDetailModal(GenDetailModal, POSSlotDetails):
     pass
 
 
+class POSSlotForeign(GenForeignKey):
+    model = POSSlot
+    label = '{name}'
+
+    def get_foreign(self, queryset, search, filters):
+        # Filter with search string
+        qsobject = Q(name__icontains=search)
+
+        queryset = queryset.filter(qsobject)
+
+        slot = filters.get('slot', None)
+
+        if slot:
+            queryset = queryset.filter(
+                Q(zone__poss__pk=slot)
+            )
+
+        return queryset
+
+
 # ###########################################
 # POSProduct
 class POSProductList(GenList):
     model = POSProduct
     extra_context = {'menu': ['pos', 'posproduct'], 'bread': [_('POS'), _('Product')]}
-    default_ordering = ["pos__name", ]
+    default_ordering = ["group_product__name", ]
 
 
 class POSProductCreate(GenCreate):
@@ -392,7 +416,26 @@ class POSProductCreate(GenCreate):
 
 
 class POSProductCreateModal(GenCreateModal, POSProductCreate):
-    pass
+    @method_decorator(login_required)
+    def dispatch(self, *args, **kwargs):
+        self.__group_pk = kwargs.get('pk', None)
+        if self.__group_pk:
+            self.form_class = POSProductFormGroup
+        return super(POSProductCreateModal, self).dispatch(*args, **kwargs)
+
+    def form_valid(self, form):
+        if self.__group_pk:
+            group_product = POSGroupProduct.objects.get(pk=self.__group_pk)
+            form.instance.group_product = group_product
+
+        try:
+            with transaction.atomic():
+                result = super(POSProductCreateModal, self).form_valid(form)
+        except IntegrityError as e:
+            errors = form._errors.setdefault("product_final", ErrorList())
+            errors.append(e)
+            return self.form_invalid(form)
+        return result
 
 
 class POSProductUpdate(GenUpdate):
@@ -415,9 +458,15 @@ class POSProductSubList(GenList):
     def __limitQ__(self, info):
         limit = {}
         pk = info.kwargs.get('pk', None)
-        limit['link'] = Q(pos__pk=pk)
+        limit['link'] = Q(group_product__pk=pk)
         return limit
 
+    def __fields__(self, info):
+        fields = []
+        fields.append(('product_final', _("Product")))
+        fields.append(('enable', _("Enable")))
+        return fields
+
 
 class POSProductDetails(GenDetail):
     model = POSProduct
@@ -619,3 +668,84 @@ class POSSession(View):
         request.session['POS_client_COMMIT'] = commit
         json_answer = json.dumps(context)
         return HttpResponse(json_answer, content_type='application/json')
+
+
+# ###########################################
+# POSGroupProduct
+class POSGroupProductList(GenList):
+    model = POSGroupProduct
+    show_details = True
+    extra_context = {'menu': ['pos', 'posgroupproduct'], 'bread': [_('POS'), _('POSGroupProduct')]}
+
+
+class POSGroupProductCreate(GenCreate):
+    model = POSGroupProduct
+    form_class = POSGroupProductForm
+
+
+class POSGroupProductCreateModal(GenCreateModal, POSGroupProductCreate):
+    pass
+
+
+class POSGroupProductUpdate(GenUpdate):
+    model = POSGroupProduct
+    form_class = POSGroupProductForm
+
+
+class POSGroupProductUpdateModal(GenUpdateModal, POSGroupProductUpdate):
+    pass
+
+
+class POSGroupProductDelete(GenDelete):
+    model = POSGroupProduct
+
+
+class POSGroupProductSubList(GenList):
+    model = POSGroupProduct
+    show_details = False
+    extra_context = {'menu': ['pos', 'POSGroupProduct'], 'bread': [_('POS'), _('POSGroupProduct')]}
+
+    def __limitQ__(self, info):
+        limit = {}
+        pk = info.kwargs.get('pk', None)
+        limit['link'] = Q(xxxxxxx__pk=pk)
+        return limit
+
+
+class POSGroupProductDetails(GenDetail):
+    model = POSGroupProduct
+    groups = POSGroupProductForm.__groups_details__()
+    tabs = [
+        {'id': 'products', 'name': _('Products'), 'ws': 'CDNX_posproducts_sublist', 'rows': 'base'},
+    ]
+
+
+class POSGroupProductDetailModal(GenDetailModal, POSGroupProductDetails):
+    pass
+
+
+class OpenCashRegister(View):
+    def get(self, request, *args, **kwargs):
+        context = {}
+        context_hardware = {'ctx': 'opening...'}
+
+        uuid_pos = self.request.session.get('POS_client_UUID', None)
+        PDV = POS.objects.filter(uuid=uuid_pos).first()
+
+        if PDV:
+            box = POSHardware.objects.filter(
+                kind=KIND_POSHARDWARE_CASH,
+                poss=PDV
+            ).first()
+            if box:
+                try:
+                    box.send(context_hardware)
+                    context = {'msg': 'OK'}
+                except IOError as e:
+                    context['error'] = e
+            else:
+                context = {'error': _('Cash register do not found')}
+        else:
+            context['error'] = _("POS not found")
+        json_answer = json.dumps(context)
+        return HttpResponse(json_answer, content_type='application/json')
